@@ -14,6 +14,7 @@ import uuid
 import libvirt
 from oslotest import base
 from six.moves import mock
+import xml.etree.ElementTree as ET
 
 from sushy_tools.emulator.drivers.libvirtdriver import LibvirtDriver
 from sushy_tools import error
@@ -60,7 +61,7 @@ class LibvirtDriverTestCase(base.BaseTestCase):
         conn_mock = libvirt_mock.return_value
         domain = mock.MagicMock()
         domain.UUIDString.return_value = self.uuid
-        conn_mock.listDefinedDomains.return_value = [domain]
+        conn_mock.listAllDomains.return_value = [domain]
         uuidstring_mock = conn_mock.lookupByName.return_value.UUIDString
         uuidstring_mock.return_value = self.uuid
         systems = self.test_driver.systems
@@ -216,7 +217,31 @@ class LibvirtDriverTestCase(base.BaseTestCase):
 
     @mock.patch('libvirt.open', autospec=True)
     @mock.patch('libvirt.openReadOnly', autospec=True)
-    def test_set_boot_mode_unknown_path(self, libvirt_mock, libvirt_rw_mock):
+    def test_set_boot_mode_known_loader(self, libvirt_mock, libvirt_rw_mock):
+        with open('sushy_tools/tests/unit/emulator/domain.xml', 'r') as f:
+            data = f.read()
+
+        conn_mock = libvirt_mock.return_value
+        domain_mock = conn_mock.lookupByUUID.return_value
+        domain_mock.XMLDesc.return_value = data
+
+        self.test_driver.set_boot_mode(self.uuid, 'Uefi')
+
+        conn_mock = libvirt_rw_mock.return_value
+        xml_document = conn_mock.defineXML.call_args[0][0]
+        tree = ET.fromstring(xml_document)
+        os_element = tree.find('os')
+        loader_element = os_element.find('loader')
+        self.assertEqual(
+            'pflash', loader_element.get('type'))
+        self.assertEqual(
+            '/usr/share/OVMF/OVMF_CODE.fd',
+            loader_element.text)
+
+    @mock.patch('libvirt.open', autospec=True)
+    @mock.patch('libvirt.openReadOnly', autospec=True)
+    def test_set_boot_mode_unknown_loader_path(
+            self, libvirt_mock, libvirt_rw_mock):
         with open('sushy_tools/tests/unit/emulator/domain.xml', 'r') as f:
             data = f.read()
 
@@ -230,6 +255,92 @@ class LibvirtDriverTestCase(base.BaseTestCase):
                 error.FishyError, self.test_driver.set_boot_mode,
                 self.uuid, 'Uefi')
 
+    @mock.patch('libvirt.open', autospec=True)
+    @mock.patch('libvirt.openReadOnly', autospec=True)
+    def test_set_boot_mode_no_os(self, libvirt_mock, libvirt_rw_mock):
+        with open('sushy_tools/tests/unit/emulator/domain.xml', 'r') as f:
+            data = f.read()
+
+        tree = ET.fromstring(data)
+        os_element = tree.find('os')
+        tree.remove(os_element)
+
+        data = ET.tostring(tree)
+
+        conn_mock = libvirt_mock.return_value
+        domain_mock = conn_mock.lookupByUUID.return_value
+        domain_mock.XMLDesc.return_value = data
+
+        self.assertRaises(
+            error.FishyError, self.test_driver.set_boot_mode,
+            self.uuid, 'Uefi')
+
+    @mock.patch('libvirt.open', autospec=True)
+    @mock.patch('libvirt.openReadOnly', autospec=True)
+    def test_set_boot_mode_many_loaders(self, libvirt_mock, libvirt_rw_mock):
+        with open('sushy_tools/tests/unit/emulator/domain.xml', 'r') as f:
+            data = f.read()
+
+        tree = ET.fromstring(data)
+        os_element = tree.find('os')
+        ET.SubElement(os_element, 'loader')
+
+        data = ET.tostring(tree)
+
+        conn_mock = libvirt_mock.return_value
+        domain_mock = conn_mock.lookupByUUID.return_value
+        domain_mock.XMLDesc.return_value = data
+
+        self.assertRaises(
+            error.FishyError, self.test_driver.set_boot_mode,
+            self.uuid, 'Uefi')
+
+    @mock.patch('libvirt.open', autospec=True)
+    @mock.patch('libvirt.openReadOnly', autospec=True)
+    def test_set_boot_mode_many_os(self, libvirt_mock, libvirt_rw_mock):
+        with open('sushy_tools/tests/unit/emulator/domain.xml', 'r') as f:
+            data = f.read()
+
+        tree = ET.fromstring(data)
+        ET.SubElement(tree, 'os')
+        ET.SubElement(tree, 'os')
+
+        data = ET.tostring(tree)
+
+        conn_mock = libvirt_mock.return_value
+        domain_mock = conn_mock.lookupByUUID.return_value
+        domain_mock.XMLDesc.return_value = data
+
+        self.assertRaises(
+            error.FishyError, self.test_driver.set_boot_mode,
+            self.uuid, 'Uefi')
+
+    @mock.patch('libvirt.open', autospec=True)
+    @mock.patch('libvirt.openReadOnly', autospec=True)
+    def test_set_boot_mode_no_type(self, libvirt_mock, libvirt_rw_mock):
+        with open('sushy_tools/tests/unit/emulator/domain.xml', 'r') as f:
+            data = f.read()
+
+        tree = ET.fromstring(data)
+        os_element = tree.find('os')
+        type_element = os_element.find('type')
+        os_element.remove(type_element)
+
+        data = ET.tostring(tree)
+
+        conn_mock = libvirt_mock.return_value
+        domain_mock = conn_mock.lookupByUUID.return_value
+        domain_mock.XMLDesc.return_value = data
+
+        self.test_driver.set_boot_mode(self.uuid, 'Uefi')
+
+        conn_mock = libvirt_rw_mock.return_value
+        xml_document = conn_mock.defineXML.call_args[0][0]
+        tree = ET.fromstring(xml_document)
+        os_element = tree.find('os')
+        # NOTE(etingof): should enforce default loader
+        self.assertIsNone(os_element.find('loader'))
+
     @mock.patch('libvirt.openReadOnly', autospec=True)
     def test_get_total_memory(self, libvirt_mock):
         conn_mock = libvirt_mock.return_value
